package dto

import (
	models "ERP_storage/models/ContractReview"
)

// ServiceItemPageReq 列表或者搜索使用结构体
type ServiceItemPageReq struct {
	Pagination        `search:"-"`
	T_name            string `form:"T_name" search:"type:contains;column:t_name;table:service_item"`                    // 服务内容
	T_service_type_id int    `form:"T_service_type_id" search:"type:exact;column:t_service_type_id;table:service_item"` // 服务类型id
	T_sale_type       string `form:"T_sale_type" search:"type:contains;column:t_sale_type;table:service_item"`          // 销售类型
}

func (m *ServiceItemPageReq) GetNeedSearch() interface{} {
	return *m
}

// ServiceItemInsertReq 增使用的结构体
type ServiceItemInsertReq struct {
	T_name            string `form:"T_name"  vd:"len($)>0;msg:'服务内容不能为空'"`        // 服务内容
	T_service_type_id int    `form:"T_service_type_id" vd:"$>0;msg:'服务类型id不能为空'"` // 服务类型id
	T_sale_type       string `form:"T_sale_type"`                                 // 销售类型
	T_model           string `form:"T_model"`                                     // 型号
	T_spec            string `form:"T_spec"`                                      // 规格
}

func (s *ServiceItemInsertReq) Generate(model *models.ServiceItem) {
	model.T_name = s.T_name
	model.T_service_type_id = s.T_service_type_id
	model.T_sale_type = s.T_sale_type
	model.T_model = s.T_model
	model.T_spec = s.T_spec
	model.T_State = 1
}

// ServiceItemUpdateReq 改使用的结构体
type ServiceItemUpdateReq struct {
	T_id              int    `form:"T_id"  example:"1"`
	T_name            string `form:"T_name"  vd:"len($)>0;msg:'服务内容不能为空'"`        // 服务内容
	T_service_type_id int    `form:"T_service_type_id" vd:"$>0;msg:'服务类型id不能为空'"` // 服务类型id
	T_sale_type       string `form:"T_sale_type"`                                 // 销售类型
	T_model           string `form:"T_model"`                                     // 型号
	T_spec            string `form:"T_spec"`                                      // 规格
}

func (s *ServiceItemUpdateReq) Generate(model *models.ServiceItem) {
	model.Id = s.T_id
	model.T_name = s.T_name
	model.T_service_type_id = s.T_service_type_id
	model.T_sale_type = s.T_sale_type
	model.T_model = s.T_model
	model.T_spec = s.T_spec
}

func (s *ServiceItemUpdateReq) GetId() interface{} {
	return s.T_id
}

// ServiceItemGetReq 获取单个的结构体
type ServiceItemGetReq struct {
	T_id int `form:"T_id"`
}

func (s *ServiceItemGetReq) GetId() interface{} {
	return s.T_id
}

// ServiceItemDeleteReq 删除的结构体
type ServiceItemDeleteReq struct {
	T_id int `form:"T_id"`
}

func (s *ServiceItemDeleteReq) GetId() interface{} {
	return s.T_id
}
