package Basic

import (
	"ERP_storage/logs"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"sync"
	"time"
)

// 仓库
type ProductClass struct {
	Id         int       `orm:"column(ID);size(11);auto;pk"`
	T_name     string    `orm:"size(256);null"`                                        // 名称
	T_State    int       `orm:"size(2);default(1)"`                                    // 0 删除（伪删除）   1 正常
	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *ProductClass) TableName() string {
	return "product_class" // 数据库名称   // ************** 替换 FormulaList **************
}

var ProductClass_list *sync.Map

func init() {
	//注册模型
	orm.RegisterModel(new(ProductClass))
	ProductClass_list = new(sync.Map)
}

type ProductClass_R struct {
	Id      int
	T_name  string // 名称
	T_State int    // 0 删除（伪删除）   1 正常
}

func ProductClassToProductClass_R(t ProductClass) (r ProductClass_R) {
	r.Id = t.Id
	r.T_name = t.T_name
	r.T_State = t.T_State
	return r
}

// 添加
func Add_ProductClass(r ProductClass) (id int64, err error) {
	o := orm.NewOrm()
	id, err = o.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	Read_ProductClass_All_Map()
	return id, err
}

// 获取 ById
func Read_ProductClass_ById(Id int) (r ProductClass, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(ProductClass))
	err = qs.Filter("Id", Id).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func Update_ProductClass(m ProductClass, cols ...string) error {
	o := orm.NewOrm()
	_, err := o.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	Read_ProductClass_All_Map()
	return nil
}

// 删除
func Delete_ProductClass(v ProductClass) error {
	o := orm.NewOrm()
	v.T_State = 0
	_, err := o.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	Read_ProductClass_All_Map()
	return err
}

// 获取列表
func Read_ProductClass_List(T_name string, page, page_z int) (r_ []ProductClass_R, cnt int64) {
	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductClass))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond = cond.And("T_State", 1).And("Id__gt", 0)

	if len(T_name) > 0 {
		cond = cond.And("T_name__icontains", T_name)
	}

	// 查询
	var r []ProductClass
	var err error
	if page_z == 9999 {
		_, err = qs.SetCond((*orm2.Condition)(cond)).OrderBy("-Id").All(&r)
	} else {
		_, err = qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond)).OrderBy("-Id").All(&r)
	}

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, ProductClassToProductClass_R(v))
	}

	return r_, cnt
}

// 获取全部
func Read_ProductClass_All_Map() {
	time.Sleep(2 * time.Second)

	logs.Println("=========== 初始化产品分类 =========")

	o := orm.NewOrm()
	for true {
		var r []ProductClass
		qs := o.QueryTable(new(ProductClass))
		_, err := qs.All(&r)
		if err != nil {
			logs.Error(lib.FuncName(), err)
		}

		for _, v := range r {
			ProductClass_list.Store(v.Id, v.T_name)
		}
		time.Sleep(1 * time.Hour)
	}
}
func Read_ProductClass_Get(Id int) string {
	// 有先加入 给全部人发消息
	v, ok := ProductClass_list.Load(Id) /*如果确定是真实的,则存在,否则不存在 */
	if ok {
		return v.(string)
	} else {
		return "未知分类"
	}
}
