package Basic

import (
	"ERP_storage/logs"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"sync"
)

// 验证项目
type VerifyItem struct {
	Id      int     `orm:"column(ID);size(11);auto;pk"`
	T_name  string  `orm:"size(256);null"`         // 名称
	T_type  int     `orm:"size(2);default(1)"`     // 类型 1-验证实施 2-报告编写
	T_price float32 `orm:"digits(12);decimals(2)"` // 单价
	T_State int     `orm:"size(2);default(1)"`     // 0 删除（伪删除）   1 正常
}

func (t *VerifyItem) TableName() string {
	return "verify_item" // 数据库名称   // ************** 替换 FormulaList **************
}

var VerifyItem_list *sync.Map

func init() {
	//注册模型
	orm.RegisterModel(new(VerifyItem))
	VerifyItem_list = new(sync.Map)

}

// 获取 ById
func Read_VerifyItem_ById(Id int) (r VerifyItem, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(VerifyItem))
	err = qs.Filter("Id", Id).Filter("T_State", 1).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 获取 ById
func Read_VerifyItem_ByT_nameT_type(T_name string, T_type int) (r VerifyItem, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(VerifyItem))
	err = qs.Filter("T_name", T_name).Filter("T_type", T_type).Filter("T_State", 1).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 获取列表
func Read_VerifyItem_List(T_type int, T_name string, page, page_z int) (r_ []VerifyItem, cnt int64) {
	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(VerifyItem))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond = cond.And("T_State", 1)

	if T_type > 0 {
		cond = cond.And("T_type", T_type)
	}

	if len(T_name) > 0 {
		cond = cond.And("T_name__icontains", T_name)
	}

	// 查询
	var r []VerifyItem
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond)).OrderBy("Id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	return r, cnt
}

// 获取全部
func Read_VerifyItem_All_Map() {
	o := orm.NewOrm()
	var r []VerifyItem
	qs := o.QueryTable(new(VerifyItem))
	_, err := qs.All(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}

	for _, v := range r {
		VerifyItem_list.Store(v.Id, v)
	}

}
func Read_VerifyItem_Get(T_id int) VerifyItem {
	// 有先加入 给全部人发消息
	v, ok := VerifyItem_list.Load(T_id) /*如果确定是真实的,则存在,否则不存在 */
	if ok {
		return v.(VerifyItem)
	}
	return VerifyItem{}
}

// 修改
func Update_VerifyItem(m VerifyItem, cols ...string) error {
	o := orm.NewOrm()
	_, err := o.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 删除
func Delete_VerifyItem(v VerifyItem) error {
	o := orm.NewOrm()
	v.T_State = 0
	_, err := o.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 添加
func Add_VerifyItem(r VerifyItem) (id int64, err error) {
	o := orm.NewOrm()
	id, err = o.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}
